/*
 * The main frame for the Command Design Pattern demo.
 * The availability of the Undo button and operation
 * is controlled by the constant {@code UNDO}.
 *
 * Created on Nov 30, 2011, 9:26:46 PM
 */
package cp.gui;

import cp.command.Command;
import cp.command.IncrementCommand;
import cp.command.ResetCommand;
import cp.model.Counter;
import java.util.Stack;

/**
 * Main frame (window) of the example application,
 * holding both the view and the controller.
 * To keep the code simple, the view and controller are tightly integrated,
 * and the controller tells the view to pull an up-to-date state from the model.
 * 
 * @author Tom Verhoeff (TU/e)
 */
public class MainFrame extends javax.swing.JFrame {

    /** Creates new form MainFrame */
    public MainFrame() {
        initComponents();
        initFrame();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanelView = new javax.swing.JPanel();
        jLabelCount = new javax.swing.JLabel();
        jTextFieldCount = new javax.swing.JTextField();
        jPanelController = new javax.swing.JPanel();
        jButtonIncrement = new javax.swing.JButton();
        jButtonReset = new javax.swing.JButton();
        jButtonUndo = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Command Pattern for Undo");

        jPanelView.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelView.setToolTipText("View");

        jLabelCount.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jLabelCount.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelCount.setText("Count");

        jTextFieldCount.setEditable(false);
        jTextFieldCount.setColumns(2);
        jTextFieldCount.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jTextFieldCount.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextFieldCount.setText("0");
        jTextFieldCount.setToolTipText("The current count");

        org.jdesktop.layout.GroupLayout jPanelViewLayout = new org.jdesktop.layout.GroupLayout(jPanelView);
        jPanelView.setLayout(jPanelViewLayout);
        jPanelViewLayout.setHorizontalGroup(
            jPanelViewLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelViewLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelViewLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabelCount, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 76, Short.MAX_VALUE)
                    .add(jTextFieldCount, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 76, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanelViewLayout.setVerticalGroup(
            jPanelViewLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelViewLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabelCount)
                .add(18, 18, 18)
                .add(jTextFieldCount, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanelController.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelController.setToolTipText("Controller");

        jButtonIncrement.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jButtonIncrement.setText("Incr.");
        jButtonIncrement.setToolTipText("Increments count by one");
        jButtonIncrement.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonIncrementActionPerformed(evt);
            }
        });

        jButtonReset.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jButtonReset.setText("Reset");
        jButtonReset.setToolTipText("Resets count to zero");
        jButtonReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResetActionPerformed(evt);
            }
        });

        jButtonUndo.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jButtonUndo.setText("Undo");
        jButtonUndo.setToolTipText("Undoes the last command");
        jButtonUndo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonUndoActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanelControllerLayout = new org.jdesktop.layout.GroupLayout(jPanelController);
        jPanelController.setLayout(jPanelControllerLayout);
        jPanelControllerLayout.setHorizontalGroup(
            jPanelControllerLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelControllerLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelControllerLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jButtonReset, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jButtonIncrement, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jButtonUndo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 89, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPanelControllerLayout.setVerticalGroup(
            jPanelControllerLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelControllerLayout.createSequentialGroup()
                .addContainerGap()
                .add(jButtonIncrement)
                .add(18, 18, 18)
                .add(jButtonReset)
                .add(18, 18, 18)
                .add(jButtonUndo)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelView, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(18, 18, 18)
                .add(jPanelController, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelController, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jPanelView, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonIncrementActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonIncrementActionPerformed
        if (! UNDO) {
            counter.increment();
        } else {
            final Command command = new IncrementCommand(counter);
            command.execute();
            undoStack.push(command);
        }
        updateFrame();
    }//GEN-LAST:event_jButtonIncrementActionPerformed

    private void jButtonResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResetActionPerformed
        if (! UNDO) {
            counter.reset();
        } else {
            final Command command = new ResetCommand(counter);
            command.execute();
            undoStack.push(command);
        }
        updateFrame();
    }//GEN-LAST:event_jButtonResetActionPerformed

    private void jButtonUndoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonUndoActionPerformed
        if (UNDO) {
            assert ! undoStack.empty() : "Undo: undo stack is empty";
            final Command command = undoStack.pop();
            command.undo();
        }
        updateFrame();
    }//GEN-LAST:event_jButtonUndoActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(MainFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonIncrement;
    private javax.swing.JButton jButtonReset;
    private javax.swing.JButton jButtonUndo;
    private javax.swing.JLabel jLabelCount;
    private javax.swing.JPanel jPanelController;
    private javax.swing.JPanel jPanelView;
    private javax.swing.JTextField jTextFieldCount;
    // End of variables declaration//GEN-END:variables

    /** Whether to provide Undo. */
    public static final boolean UNDO = true;
    
    /** The counter (model). */
    private Counter counter;
    
    /** The undo stack. */
    private Stack<Command> undoStack;

    /** Initialize this frame. */
    private void initFrame() {
        counter = new Counter();
        undoStack = new Stack<Command>();
        updateFrame();
    }
    
    /** Update this frame, both controller and view. */
    private void updateFrame() {
        if (! UNDO) {
            jButtonUndo.setEnabled(false);
            //jButtonUndo.setVisible(false);
            jButtonUndo.setToolTipText(
                    "To enable Undo, recompile with UNDO = true");
        } else {
            if (undoStack.empty()) {
                jButtonUndo.setToolTipText("Nothing to undo");
            } else {
                jButtonUndo.setToolTipText("Undo last command of " +
                        undoStack.size());
            }
            jButtonUndo.setEnabled(! undoStack.empty());
        }
        jTextFieldCount.setText("" + counter.getCount());
    }
    
}
